defmodule DecentralisedBookIndexWeb.BookLive.FormComponent do
  use DecentralisedBookIndexWeb, :live_component

  @impl true
  def render(assigns) do
    ~H"""
    <div>
      <.header>
        {@title}
        <:subtitle>Use this form to manage book records in your database.</:subtitle>
      </.header>

      <.simple_form
        for={@form}
        id="book-form"
        phx-target={@myself}
        phx-change="validate"
        phx-submit="save"
      >
        <%= if @form.source.type == :create do %>
          <.input field={@form[:title]} type="text" label="Title" />
          <.input
            field={@form[:description]}
            type="text"
            label="Description"
          />
          <.bids_inputs form={@form} myself={@myself} />
          <.author_roles_inputs form={@form} myself={@myself} />
          <.input field={@form[:format]} type="text" label="Format" /><.input
            field={@form[:language]}
            type="text"
            label="Language"
          /><.input field={@form[:page_count]} type="number" label="Page count" /><.input
            field={@form[:published]}
            type="date"
            label="Published"
          />
        <% end %>
        <%= if @form.source.type == :update do %>
        <% end %>

        <:actions>
          <.button phx-disable-with="Saving...">Save Book</.button>
        </:actions>
      </.simple_form>
    </div>
    """
  end

  def bids_inputs(assigns) do
    ~H"""
    <h2>Book Ids</h2>

    <table class="w-full">
      <thead class="border-b border-zinc-100">
        <tr>
          <th class="text-left font-medium text-sm pb-1 px-3">Type</th>
          <th class="text-left font-medium text-sm pb-1 px-3" colspan="2">Id</th>
        </tr>
      </thead>
      <tbody phx-hook="bidSort" id="bidSort" phx-target={@myself}>
        <.inputs_for :let={bid_form} field={@form[:bids]}>
          <tr data-id={bid_form.index}>
            <td class="px-3 w-24">
              <label for={bid_form[:type].id} class="hidden">Type</label>
              <.input field={bid_form[:type]} />
            </td>
            <td class="px-3">
              <label for={bid_form[:bid].id} class="hidden">Id</label>
              <.input field={bid_form[:bid]} />
            </td>
          </tr>
        </.inputs_for>
      </tbody>
    </table>

    <.button_link phx-click="add-bid" phx-target={@myself} kind="primary" size="sm" inverse>
      Add Book Id.
    </.button_link>
    """
  end

  @impl true
  def handle_event("add-bid", params, socket) do

    socket =
      update(socket, :form, fn form ->
        AshPhoenix.Form.add_form(form, :bids)
      end)

    {:noreply, socket}
  end

  @impl true
  def handle_event("remove-bid", %{"path" => path}, socket) do
    socket =
      update(socket, :form, fn form ->
        AshPhoenix.Form.remove_form(form, path)
      end)

    {:noreply, socket}
  end

  @impl true
  def handle_event("reorder-bids", %{"order" => order}, socket) do
    socket =
      update(socket, :form, fn form ->
        AshPhoenix.Form.sort_forms(form, [:bids], order)
      end)

    {:noreply, socket}
  end

  def author_roles_inputs(assigns) do
    ~H"""
    <h2>Authors</h2>

    <table class="w-full">
      <thead class="border-b border-zinc-100">
        <tr>
          <th class="text-left font-medium text-sm pb-1 px-3">Author</th>
          <th class="text-left font-medium text-sm pb-1 px-3">Role</th>
        </tr>
      </thead>
      <tbody phx-hook="authorRoleSort" id="authorRoleSort" phx-target={@myself}>
        <.inputs_for :let={author_roles_form} field={@form[:author_roles]}>
          <tr data-id={author_roles_form.index}>
            <td class="px-3">
              <label for={author_roles_form[:author_id].id} class="hidden">Type</label>
              <.input field={author_roles_form[:author_id]} />
            </td>

            <td class="px-3">
              <label for={author_roles_form[:role].id} class="hidden">Id</label>
              <.input field={author_roles_form[:role]} />
            </td>
          </tr>
        </.inputs_for>
      </tbody>
    </table>

    <.button_link phx-click="add-author-role" phx-target={@myself} kind="primary" size="sm" inverse>
      Add Author
    </.button_link>
    """
  end

  @impl true
  def handle_event("add-author-role", params, socket) do
    socket =
      update(socket, :form, fn form ->
        AshPhoenix.Form.add_form(form, :author_roles)
      end)

    {:noreply, socket}
  end

  @impl true
  def handle_event("remove-author-role", %{"path" => path}, socket) do
    socket =
      update(socket, :form, fn form ->
        AshPhoenix.Form.remove_form(form, path)
      end)

    {:noreply, socket}
  end

  @impl true
  def handle_event("reorder-author-role", %{"order" => order}, socket) do
    socket =
      update(socket, :form, fn form ->
        AshPhoenix.Form.sort_forms(form, [:author_roles], order)
      end)

    {:noreply, socket}
  end

  @impl true
  def update(assigns, socket) do
    {:ok,
     socket
     |> assign(assigns)
     |> assign_form()}
  end

  @impl true
  def handle_event("validate", %{"book" => book_params}, socket) do
    {:noreply, assign(socket, form: AshPhoenix.Form.validate(socket.assigns.form, book_params))}
  end

  def handle_event("save", %{"book" => book_params}, socket) do
    case AshPhoenix.Form.submit(socket.assigns.form, params: book_params) do
      {:ok, book} ->
        notify_parent({:saved, book})

        socket =
          socket
          |> put_flash(:info, "Book #{socket.assigns.form.source.type}d successfully")
          |> push_patch(to: socket.assigns.patch)

        {:noreply, socket}

      {:error, form} ->
        {:noreply, assign(socket, form: form)}
    end
  end

  defp notify_parent(msg), do: send(self(), {__MODULE__, msg})

  defp assign_form(%{assigns: %{book: book}} = socket) do
    form =
      if book do
        AshPhoenix.Form.for_update(book, :update, as: "book", actor: socket.assigns.current_user)
      else
        AshPhoenix.Form.for_create(DecentralisedBookIndex.Metadata.Book, :create,
          transform_params: fn _form, params, _context ->
            Map.put(params, "author_roles", ["d880b385-5df6-4127-a49c-16f9070ea066"])
            Map.put(params, "publishers", ["9ca6f110-8ef4-494b-9979-f5b016d0f45e"])
          end,
          as: "book",
          actor: socket.assigns.current_user
        )
      end

    assign(socket, form: to_form(form))
  end
end
